package ru.ifmo.ga.ca;

import ru.ifmo.ga.ca.configuration.Configuration;
import ru.ifmo.ga.ca.configuration.ConfigurationFactory;
import ru.ifmo.ga.ca.tournament.Result;
import ru.ifmo.ga.ca.tournament.Tournament;

import java.util.HashSet;
import java.lang.reflect.Field;

import ext.bmin.MinimizedTable;

/**
 * User: Yuri D. Bedny
 * Date: Apr 9, 2006
 * Time: 4:53:32 PM
 */

public class Utils {

    public static String arrToString(boolean []arr) {
        StringBuilder sb = new StringBuilder();
        for (boolean b : arr) {
            sb.append(b ? "1" : "0");
        }
        return sb.toString();
    }

    public static int diffFromKnown(String s) throws Exception {
        int min = s.length();
        String rule = "";
        Field[] fields = KnownCARepository.class.getFields();
        for (Field field : fields) {
            String v = (String)field.get(null);
            int m = Utils.diff(v, s);
            if (m < min) {
//                rule = field.getName();
                min = m;
            }
        }
//        System.out.println(rule);
        return min;           
    }

    public static int diff(String s1, String s2) throws Exception {
        return diff(getRule(s1), getRule(s2));
    }

    public static int diff(Rule r1, Rule r2) {
        int t = 0;
        for (int i = 0; i < r1.getLength(); i++) {
            if (r1.getBit(i) != r2.getBit(i)) {
                t++;
            }
        }
        return t;
    }

    public static long grey(long x) {
        return x ^ (x >> 1);
    }

    public static long inv(final int size, final long x) {
        return ~x & (((long)1 << size) - 1);
    }

    public static long cyclicShiftLeft(final int size, final long x) {
        final long mask = (long)1 << (size - 1);
        final long last = (mask & x) >> (size - 1);
        return ((x & (mask - 1)) << 1) + last;
    }

    private static SimpleCA getCA(Configuration c, Rule rule) {
        SimpleCA ca = new SimpleCA();
        ca.setSize(c.size());
        ca.setConfiguration(c);
        ca.setRule(rule);
        return ca;
    }

    public static String isPeriodic(Configuration c, Rule rule, long max) {
        SimpleCA ca = getCA(c, rule);
        int card = 0;
        HashSet<Configuration> hash = new HashSet<Configuration>();
        card = ca.getConfiguration().cardinality();
        int maxd = 0;
        int d;
        for (int i = 0; i < max; i++) {
            if (hash.contains(ca.getConfiguration())) {
                if (ca.getConfiguration().cardinality() == 0)
                    return "1, " + i + ", " + maxd;
                else if (ca.getConfiguration().cardinality() == ca.getSize())
                    return "0, " + i + ", " + maxd;
                else
                    return "per";
            } else {
                hash.add(ca.getConfiguration());
            }
            if (ca.getConfiguration().cardinality() > 2 && ca.getConfiguration().cardinality() < ca.getSize() - 2) {
                d = Math.abs(card - ca.getConfiguration().cardinality());
                if (d > maxd)
                    maxd = d;
                card = ca.getConfiguration().cardinality();
            }
            ca.next();
        }
        return "non";
    }

    public static boolean equivalent(Configuration c1, Configuration c2, Rule rule, long max) {
        if (c1.size() != c2.size())
            return false;

        SimpleCA ca1 = getCA(c1, rule);
        SimpleCA ca2 = getCA(c2, rule);

        for (int i = 0; i < max; i++) {
            if (ca1.getConfiguration().equals(ca2.getConfiguration())) {
                return true;
            }
            if (ca1.isStopped() && ca2.isStopped())
                return false;
            ca1.next();
            ca2.next();
        }

        return false;

    }

    public static String minRuleExpr(String rule) throws Exception {
        Rule r = getRule(rule);
        MinimizedTable mt = new MinimizedTable(
                r.toCNF(new char[]{'a', 'b', 'c', 'd', 'e', 'f', 'g'}), null
        );
        return mt.toString();
    }

    public static String toBinaryString(int l, long x) {
        StringBuilder sb = new StringBuilder();
        sb.append(Long.toBinaryString(x));
        while (sb.length() < l) {
            sb.insert(0, "0");
        }
        return sb.toString();
    }

    public static Rule getRule(String rule) throws Exception {
        Rule r = new Rule();
        r.setRadius(3);
        r.parseRule(rule);
        return r;
    }

    public static boolean []getArr(int l, int x) {
        return new ConfigurationFactory(ConfigurationFactory.Implementation.BooleanArray).create(l, x).toBooleanArray();
    }


    public static Result testRule(String s, int testCases) throws Exception {
        Rule r= new Rule();
        r.setRadius(3);
        r.parseRule(s);
        return testRule(r, testCases);

    }

    public static Result testRule(String s) throws Exception {
        return testRule(s, 1000);
    }

    public static Result testRule(Rule r) {
        return testRule(r, 1000);
    }

    public static Result testRule(Rule r, int testCases) {
        Tournament t = new Tournament();
        final int SIZE = 149;
        t.getSettings().setRuns(testCases);
        t.getSettings().setSteps(SIZE * 2);
        t.getSettings().setSize(SIZE);
        t.getRules().add(r);
        return t.play().getResults().get(0);
    }

    public static long flipBit(long x, long pos) {
        final long mask = 1 << pos;
        return x ^ mask;
    }

    public static Rule randomRule() {
        Rule r = new Rule();
        r.setRadius(3);
        for (int i = 0; i < r.getLength(); i++) {
            r.setBit(i, Math.random() < 0.5);
        }
        return r;
    }

}
