package ru.ifmo.ga.ca.configuration;

/**
 * User: Yuri D. Bedny
 * Date: Apr 14, 2006
 * Time: 2:22:42 PM
 */

abstract class AbstractConfiguration implements Configuration {

    protected final int size;

    public AbstractConfiguration(int size, long x) throws IllegalArgumentException {
        this(size);
        final long max = (long)1 << (size + 1) - 1;
        if (x >= max)
            throw new IllegalArgumentException("x > 2^size - 1");
        int i = 0;
        while (x > 0) {
            set(i++, (x % 2) == 1);
            x >>= 1;
        }
    }

    public AbstractConfiguration(String s) throws IllegalArgumentException {
        this(s.length());
        for (int i = 0; i < s.length(); i++) {
            char bit = s.charAt(i);
            if (bit != '0' && bit != '1') {
                throw new IllegalArgumentException("Invalid character found at position " + i + ". String must contain only '0's and '1's.");
            }
            set(s.length() - i - 1, bit == '1');
        }
    }

    public AbstractConfiguration(int size) {
        this.size =  size;
        init();
    }

    public AbstractConfiguration(boolean[] f) {
        this(f.length);
        for (int i = 0; i < f.length; i++) {
            set(i, f[i]);
        }
    }

    public boolean isClassified(boolean allones) {
        return (allones && cardinality() == size) || (!allones && cardinality() == 0);
    }

    public boolean isOnesMajority() {
        return cardinality() * 2 > size;
    }

    public int size() {
        return size;
    }

    protected abstract void init();

    public abstract int cardinality();

    public abstract long toLong();

    public String toString() {
        StringBuilder sb = new StringBuilder();
        for (int i = size - 1; i >= 0; i--) {
            sb.append(get(i) ? '1' : '0');
        }
        return sb.toString();
    }

    public Configuration clone() {
        AbstractConfiguration result;
        try {
            result = (AbstractConfiguration) super.clone();
        } catch (CloneNotSupportedException e) {
            throw new InternalError();
        }
        return result;
    }

    public int hashCode() {
        return toString().hashCode();
    }

    public boolean equals(Object obj) {
        if (obj == null)
            return false;

        if (!(obj instanceof Configuration))
            return false;

        return isCyclicEquals((Configuration) obj, 0);
    }

    private boolean isCyclicEquals(Configuration cfg, int shift) {
        if (cfg.size() != size)
            return false;

        for (int i = 0; i < size; i++)
            if (cfg.get(i) != get((i + shift) % size))
                return false;

        return true;
    }

    public boolean isCyclicEquals(Configuration cfg) {//todo: very stupid implementation
        for (int sh = 0; sh < size; sh++)
            if (isCyclicEquals(cfg, sh))
                return true;

        return false;
    }

    public int diameter() {//todo: implement
        return 0;  //To change body of implemented methods use File | Settings | File Templates.
    }
}
