package ru.ifmo.ga.ca.configuration;

import java.util.Arrays;

/**
 * User: Yuri D. Bedny
 * Date: Apr 14, 2006
 * Time: 2:52:41 PM
 */

class BooleanArrayConfiguration extends AbstractConfiguration {

    private boolean EMPTY[];
    private boolean bits[];

    // cached
    private int card = -1;

    protected void init() {
        EMPTY = new boolean[size];
        bits = new boolean[size];
        card = 0;
    }

    public BooleanArrayConfiguration(boolean b[]) {
        super(b.length);
        System.arraycopy(b, 0, bits, 0, size);
    }

    public BooleanArrayConfiguration(String s) throws IllegalArgumentException {
        super(s);
    }

    public BooleanArrayConfiguration(int size, long x) throws IllegalArgumentException {
        super(size, x);
    }

    public BooleanArrayConfiguration(int size) {
        super(size);
    }

    public long toLong() {
        long sum = 0;
        final int l = Math.min(64, size);
        for (int i = l - 1; i >= 0; i--) {
            sum <<= 1;
            sum += bits[i] ? 1 : 0;
        }
        return sum;
    }

    public void clear() {
       System.arraycopy(EMPTY, 0, bits, 0, size);
       card = 0;
    }

    public boolean get(int i) {
        return bits[i];
    }

    public void set(int i, boolean b) {
        if (card != -1 && !bits[i] && b) {
            card++;
        } else if (card != -1 && bits[i] && !b) {
            card--;
        }

        bits[i] = b;
    }

    public int cardinality() {
        if (card != -1) {
            return card;
        }
        int r = 0;
        for (boolean b : bits) {
            if (b) {
                r++;
            }
        }
        card = r;
        return r;
    }

    public Configuration clone() {
        BooleanArrayConfiguration c = (BooleanArrayConfiguration) super.clone();
        c.bits = new boolean[size];
        c.card = card;
        System.arraycopy(bits, 0, c.bits, 0, size);
        return c;
    }

    public boolean[] toBooleanArray() {
        boolean[] b = new boolean[size];
        System.arraycopy(bits, 0, b, 0, size);
        return b;
    }

    public boolean equals(Object obj) {
        if (!(obj instanceof BooleanArrayConfiguration))
            return super.equals(obj);

        BooleanArrayConfiguration cfg = (BooleanArrayConfiguration) obj;
        return Arrays.equals(bits, cfg.bits);
    }
}
