package ru.ifmo.ga.ca.dct;

import ru.ifmo.ga.fx.Module;
import ru.ifmo.ga.fx.Server;
import ru.ifmo.ga.fx.Repository;
import ru.ifmo.ga.fx.UploadResult;
import ru.ifmo.ga.fx.desire.MostDifferentPointInDeltaDesire;
import ru.ifmo.ga.ca.Rule;
import ru.ifmo.ga.ca.dct.CAPoint;

import java.rmi.registry.Registry;
import java.rmi.registry.LocateRegistry;
import java.rmi.RemoteException;
import java.rmi.NotBoundException;
import java.util.Date;

public abstract class CAModule implements Module {

    protected Repository repository;
    protected Server server;
    private CAPoint sample;

    public String getName() {
        return getClass().getCanonicalName();
    }

    protected CAModule() {
        this(null);
    }

    protected CAModule(String host) {
        try {
            Registry registry = LocateRegistry.getRegistry(host);
            server = (Server) registry.lookup("DCTServer");
            repository = server.getRepository(CARepository.DEFAULT_CA_REPOSITORY_NAME);
            sample = new CAPoint();
        } catch (RemoteException e) {
            System.out.println("Can't connect to DCT server (" + e.getMessage() + ").");
        } catch (NotBoundException e) {
            System.out.println("Can't connect to DCT server (" + e.getMessage() + ").");
        }
    }

    public Repository getRepository() {
        return repository;
    }

    public Server getServer() {
        return server;
    }



    protected Rule download(Rule r) {
        if (repository == null) {
            return null;
        }

        sample.setRule(r);
        try {
            CAPoint caPoint = (CAPoint)repository.downloadPoint(new MostDifferentPointInDeltaDesire(getName(), sample, 0, 0.1));
            System.out.println("CAPoint downloaded: " + (caPoint == null ? "no" : caPoint.getMetadata().getValue()));
            if (caPoint != null) {
                System.out.println(caPoint.getRule().toString());
            }
            return caPoint == null ? null : caPoint.getRule();
        } catch (RemoteException e) {
            System.out.println("Can't download CAPoint!");
            return null;
        }
    }

    protected void upload(Rule r, int test_cases, double value) {
        if (repository == null) {
            return;
        }

        CAPoint caPoint = new CAPoint();
        caPoint.setRule(r);
        caPoint.getMetadata().setValue(value);
        caPoint.getMetadata().setTestCases(test_cases);
        caPoint.getMetadata().setDate(new Date());
        caPoint.getMetadata().setClientName(getName());
        try {

            UploadResult uploadResult = repository.uploadPoint(caPoint);
            if (uploadResult == UploadResult.OK) {
                System.out.println("----- CAPoint was uploaded to server. ------ ");
                System.out.println("Estimated value: " + value);
            } else {
                System.out.println(uploadResult);
            }
        } catch (RemoteException e) {
            System.out.println("----- CAPoint was NOT uploaded to server! ----- ");
        }

    }
}
