package ru.ifmo.ga.ca.dct.modules;

import ru.ifmo.ga.ca.*;
import ru.ifmo.ga.ca.tree.Tree;
import ru.ifmo.ga.ca.configuration.ConfigurationFactory;

/**
 * User: Yuri D. Bedny
 * Date: May 23, 2006
 * Time: 10:28:18 PM
 */

public class GEPUtils {

    public static void express(Rule r, int ch[][], int headLength) {
        final int l = 2 * r.getRadius() + 1;
        final int max = 1 << l;
        for (int j = 0; j < max; j++) {
            boolean b = doCalc(ch, getBoolArray(j, l), headLength);
            r.setBit(j, b);
        }
    }

    private static boolean doCalc(int[] g, boolean[] values, int headLength) {

        // start position
        int s = 0;
        for (int i = headLength - 1; i >= 0 ; i--) {
            if (g[i] == GEP.START_CODON) {
                s = i + 1;
                break;
            }
        }

        int gene[] = new int[g.length - s];
        System.arraycopy(g, s, gene, 0, gene.length);

        int[][] tree = new int[gene.length][3];
        int[] queue = new int[gene.length];
        int rp = 0;
        int wp = 0;
        queue[wp++] = 0;
        while (rp < wp) {
            int i = queue[rp++];
            if (gene[i] < 0) {                  // is function
                assert (gene[i] != GEP.START_CODON); // should be already removed
                for (int k = 0; k < 3; k++) {
                    queue[wp] = wp;
                    tree[i][k] = wp;
                    wp++;
                }
            }
        }

        return calcTree(tree, gene, 0, values);
    }

    private static boolean link(boolean x, boolean y, boolean z) {
        return y ?  x : z;
    }

    private static boolean doCalc(int[][] chromosome, boolean[] values, int headLength) {
        boolean x = doCalc(chromosome[0], values, headLength);
        boolean y = doCalc(chromosome[1], values, headLength);
        boolean z = doCalc(chromosome[2], values, headLength);
        return link(x, y, z);
    }

    private static final ConfigurationFactory factory = new ConfigurationFactory(ConfigurationFactory.Implementation.BooleanArray);

    private static boolean [] getBoolArray(int i, int l) {
        return factory.create(l, i).toBooleanArray();
    }

    private static boolean calcTree(int[][] tree, int[] gene, int i, boolean[] values) {

        // TERMINALS

        if (gene[i] >= 0) {
            if (gene[i] == GEP.TRUTH_TERM) {
                return true;
            } else if (gene[i] == GEP.FALSE_TERM) {
                return false;
            } else {
                return (gene[i] < 7) ? values[gene[i]] : !values[gene[i] - 7];
            }
        }

        // FUNCTIONS

        if (gene[i] == -1) {// IF
            return                      calcTree(tree, gene, tree[i][0], values) ?
                    calcTree(tree, gene, tree[i][1], values) : calcTree(tree, gene, tree[i][2], values);
        }

        if (gene[i] == -2) { // M
            boolean x = calcTree(tree, gene, tree[i][0], values);
            boolean y = calcTree(tree, gene, tree[i][1], values);
            boolean z = calcTree(tree, gene, tree[i][2], values);
            return x && y || x && z || y && z;
        }

        if (gene[i] == -3) { // OR
            return calcTree(tree, gene, tree[i][0], values) || calcTree(tree, gene, tree[i][1], values);
        }


        if (gene[i] == -4) { // AND
            return calcTree(tree, gene, tree[i][0], values) && calcTree(tree, gene, tree[i][1], values);
        }

        if (gene[i] == -5) { // NOT
            return !calcTree(tree, gene, tree[i][0], values);
        }

        if (gene[i] == -6) { // NOR
            return !(calcTree(tree, gene, tree[i][0], values) || calcTree(tree, gene, tree[i][1], values));
        }

        if (gene[i] == -7) { // NAND
            return !(calcTree(tree, gene, tree[i][0], values) && calcTree(tree, gene, tree[i][1], values));
        }

        if (gene[i] == -8) { // XOR
            return calcTree(tree, gene, tree[i][0], values) ^ calcTree(tree, gene, tree[i][1], values);
        }

        throw new RuntimeException(String.valueOf(gene[i]));
    }

    public static void treeToGene(Tree tree, int []gene) {
        Object[] queue = new Object[gene.length];
        int wp = 0;
        int rp = 0;
        queue[wp++] = tree;
        int j = 0;
        while (rp < wp) {
            Object o = queue[rp++];
            if (o instanceof Tree) {
                Tree t = (Tree) o;
                int x = getVar(t);
                if (t.isTerminal()) {
                    gene[j++] = x;
                } else {
                    gene[j++] = -1;          // IF
                    queue[wp++] = x;
                    queue[wp++] = t.getThen();
                    queue[wp++] = t.getElse();
                }
            } else {
                gene[j++] = (Integer)o;
            }
        }
    }

    private static int getVar(Tree t) {
        if (t.isTerminal()) {
            return t.isTruth() ? GEP.TRUTH_TERM : GEP.FALSE_TERM;
        }
        return 6 - t.getVarNum();
    }

    static final char[] VARS = new char[]{'a', 'b', 'c', 'd', 'e', 'f', 'g'};

    public static void ruleToChromosome(Rule r, int[][] chromosome) {
        treeToChromosome(new Tree(r, VARS), chromosome);
    }

    public static void treeToChromosome(Tree tree, int[][] chromosome) {
        if (tree.isTerminal()) {
            chromosome[1][0] = GEP.TRUTH_TERM;
            chromosome[0][0] = tree.isTruth() ? GEP.TRUTH_TERM : GEP.FALSE_TERM;
            return;
        }

        chromosome[1][0] = getVar(tree);
        treeToGene(tree.getThen(), chromosome[0]);
        treeToGene(tree.getElse(), chromosome[2]);
    }

    public static void copy(int[][] p1, int[][] p2) {
        for (int k = 0; k < p1.length; k++) {
            System.arraycopy(p1[k], 0, p2[k], 0, p1[k].length);
        }
    }

}
