package ru.ifmo.ga.ca.gui;

import javax.swing.*;
import java.awt.*;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

/**
 * User: Yuri D. Bedny
 * Date: Apr 8, 2006
 * Time: 1:37:59 PM
 */

public class CAField extends JComponent implements CAModelListener, MouseMotionListener, MouseListener {
    private CAModel model = new DefaultCAModel();

    private Color borderColor = Color.BLACK;
    private Color trueColor = Color.GRAY;
    private Color falseColor = Color.WHITE;
    private Color highlightColor = Color.BLUE;
    private Color centerColor = Color.RED;

    private int highlightIndex = -1;

//    private class CAToolTip extends JToolTip {
//        private final int w;
//
//        public CAToolTip(int w) {
//            this.w = w;
//            setLocation(20, 20);
//            setPreferredSize(new Dimension(w, w));
//        }
//
//        protected void paintComponent(Graphics g) {
//            super.paintComponent(g);
//            g.setColor(Color.WHITE);
//            g.fillRect(0, 0, w, w);
//        }
//    }

    public CAModel getModel() {
        return model;
    }

    public void caChanged(CAModelEvent e) {
        repaint();
    }

//    public JToolTip createToolTip() {
//        JToolTip toolTip = (highlightIndex != - 1) ?
//                new CAToolTip(Math.min(getWidth() / model.getCa().size(), getHeight())) :
//                new JToolTip();
//        return toolTip;
//    }

    public CAField() {
//        setToolTipText("");
        model.addCAModelListener(this);
        addMouseMotionListener(this);
        addMouseListener(this);
        setMinimumSize(new Dimension(200, 20));
        setPreferredSize(new Dimension(401, 70));
    }

    private void drawCell(Graphics g, final int i) {
        final int w = Math.min(getWidth() / model.getCa().getSize(), getHeight());
        final boolean b = model.getCa().getConfiguration().get(i);

        /**
         * Fill rect
         */
        g.setColor(b ? trueColor : falseColor);
        g.fillRect(i * w + 1, 1, w - 1, w - 1);

        /**
         * Draw rect
         */
        g.setColor(borderColor);
        g.drawRect(i * w, 0, w, w);

        /**
         * Highlight
         */
        if (isHighlight(i)) {
            g.setColor(i == highlightIndex ? centerColor : highlightColor);
            g.fillRect(i * w, w + 2, w + 1, w / 10);
        }
    }

    private boolean isHighlight(int i) {
        if (highlightIndex == -1)
            return false;

        final int r = model.getCa().getRule().getRadius();
        final int d = Math.abs(highlightIndex - i);
        return d <= r || d >= model.getCa().getSize() - r;
    }

    protected void paintComponent(Graphics g) {
        super.paintComponent(g);
        for (int i = 0; i < model.getCa().getSize(); i++) {
            drawCell(g, i);
        }
    }

    public void mouseDragged(MouseEvent e) {
        //To change body of implemented methods use File | Settings | File Templates.
    }

    public void mouseMoved(MouseEvent e) {
//        ToolTipManager.sharedInstance().mouseMoved(e);
        final int w = Math.min(getWidth() / model.getCa().getSize(), getHeight());

        int ind = (e.getX() < w * model.getCa().getSize() && e.getY() < w) ?
                (e.getX() / (getWidth() / model.getCa().getSize())) : -1;

        if (ind != highlightIndex) {
            highlightIndex = ind;
//            ToolTipManager.sharedInstance().mouseExited(e);
            repaint();
        }
    }


    public void mouseClicked(MouseEvent e) {
        //To change body of implemented methods use File | Settings | File Templates.
    }

    public void mousePressed(MouseEvent e) {
        //To change body of implemented methods use File | Settings | File Templates.
    }

    public void mouseReleased(MouseEvent e) {
        //To change body of implemented methods use File | Settings | File Templates.
    }

    public void mouseEntered(MouseEvent e) {

        //To change body of implemented methods use File | Settings | File Templates.
    }

    public void mouseExited(MouseEvent e) {
        if (highlightIndex != -1) {
            highlightIndex = -1;
            repaint();
        }
    }
}
