package ru.ifmo.ga.ca;

import junit.framework.TestCase;
import ru.ifmo.ga.ca.configuration.Configuration;
import ru.ifmo.ga.ca.configuration.ConfigurationFactory;
import ru.ifmo.ga.ca.tournament.Result;
import ru.ifmo.ga.ca.tournament.Tournament;

import java.util.ArrayList;

/**
 * User: Yuri D. Bedny
 * Date: May 1, 2006
 * Time: 8:43:07 PM
 */

public class AttractorTest extends TestCase {

    private static boolean was(Rule rule, ArrayList<Configuration> cfgs, Configuration cfg, int t) {
        for (int i = 0; i < t; i++)
            if (cfgs.get(i).isCyclicEquals(cfg))
                return true;

        for (int i = 0; i < t; i++)
            if (Utils.equivalent(cfgs.get(i), cfg, rule, 10000))
                return true;

        return false;
    }

    public static void testx() throws Exception {
        SimpleCA ca = new SimpleCA();
        ca.setSize(149);
        Rule r = new Rule();
        r.setRadius(3);
        r.parseRule(KnownCARepository.JP_2);
        Configuration c = new ConfigurationFactory(ConfigurationFactory.Implementation.BooleanArray).create(149);
        for (int i = 0; i < 149; i++) {
            c.set(i, i < 75);
        }
        assertFalse(r.getBit(0));
        assertFalse(r.getBit(8));
        assertTrue(r.getBit(127));
        assertTrue(r.getBit(119));
        c.set(23, false);

        ca.setConfiguration(c);
        ca.setRule(r);

        assertEquals(ca.getConfiguration().isOnesMajority(), c.isOnesMajority());
        for (int i = 0; i < 1000000; i++) {
            assertEquals(Integer.toString(i), ca.getConfiguration().isOnesMajority(), c.isOnesMajority());
            if (ca.isStopped()) {
                break;
            }
            ca.next();
        }

        assertEquals(ca.getConfiguration().isOnesMajority(), c.isOnesMajority());
    }

    public static void main(String[] args) throws Exception {
        Tournament t = new Tournament();
        t.getSettings().setRuns(1000);
        t.getSettings().setSteps(300);
        t.getSettings().setSize(149);

        Rule r1 = new Rule();
        r1.setRadius(3);
        r1.parseRule(KnownCARepository.KOZA);
        System.out.println(r1.cardinality() * 1.0 / 128);
        t.getRules().add(r1);

//        Rule r2 = new Rule();
//        r2.setRadius(3);
//        r2.parseRule(KnownCARepository.GKL);
//        t.getRules().add(r2);

        ArrayList<Result> res =  t.play().sort();
        out(res.get(0));
//        System.out.println("");
//        out(res.get(1));
    }

    public void testOneFailed() throws Exception {
        Configuration c = new ConfigurationFactory(ConfigurationFactory.Implementation.BooleanArray).create("01111001010011000000111000001111101001101001011100100101100100101110101001001010001101111011110011010100101001001101011001011010010010010110010111110");
                                        //"00011110110111100110000010101111100011010100000011000001100011000100111101111010001001010100111100101111110100111010100110000000000100111001000110111"
        final SimpleCA ca = new DefaultCABuilder().build(KnownCARepository.GKL);
        ca.setConfiguration(c);

        while (!ca.isStopped()) {
            System.out.print(ca.getConfiguration().cardinality() + " ");
            ca.next();
        }
//        SwingUtilities.invokeLater(new Runnable() {
//            public void run() {
//                new CAFrame(ca);
//            }
//        });
//
//        for (int i = 0; i < 200; i++) {
//            ca.next();
//            if (ca.isStopped())
//                break;
//            Thread.sleep(1000);
//        }

    }
    private static void out(Result result) {
//        System.out.println(result);
//        ArrayList<ICContext> cfgs = result.getLostCfgs();
//        ArrayList<Configuration> list = new ArrayList<Configuration>();
//        System.out.println(cfgs.size());
//        for (int i = 0; i < cfgs.size(); i++) {
//            if (!was(result.getRule(), cfgs, cfgs.get(i).getCfg(), i)) {
//                System.out.println(Utils.isPeriodic(cfgs.get(i), result.getRule(), 1000000) +  ", " + cfgs.get(i));
//            }
//            SimpleCA ca = new DefaultCABuilder().build(result.getRule());
//            ca.setConfiguration(cfgs.get(i));
//            while (ca.getConfiguration().cardinality() > 2 && ca.getConfiguration().cardinality() < 147) {
//                ca.next();
//            }
//            if (ca.getConfiguration().cardinality() < 10) {
//                list.add(ca.getConfiguration());
//            }
//        }

//        System.out.println("-----------------");
//        for (int i = 0; i < list.size(); i++) {
//            if (!was(result.getRule(), list, list.get(i), i))
//                System.out.println(list.get(i));
//        }
    }
}
