package ru.ifmo.ga.ca;

import junit.framework.TestCase;
import ru.ifmo.ga.ca.Rule;
import ru.ifmo.ga.ca.Utils;
import ru.ifmo.ga.ca.KnownCARepository;
import ru.ifmo.ga.ca.SimpleCA;
import ru.ifmo.ga.ca.configuration.Configuration;
import ru.ifmo.ga.ca.configuration.ConfigurationFactory;
import ru.ifmo.ga.ca.tournament.Tournament;
import ru.ifmo.ga.ca.tournament.Results;
import ru.ifmo.ga.ca.tournament.Result;
import ru.ifmo.ga.ca.tournament.ICContext;

/**
 * User: Yuri D. Bedny
 * Date: Apr 9, 2006
 * Time: 5:11:51 PM
 */

public class GKLTest extends TestCase {
    private static Rule rule;

    public void setUp() throws Exception {
        rule = Utils.getRule(KnownCARepository.GKL);
    }


    public void testSymmetry() {
        System.out.println(rule.toTable());            
    }

    public void testMonotomous() {
        assertFalse(rule.isMonotonous());
    }

    public void testWashing() {
        assertFalse(rule.isWashing());
    }

    public void testFault() {
        final int INIT_SIZE = 501;
        final int SIZE = 5000;

        Tournament t = new Tournament();
        t.getRules().add(rule);
        t.getSettings().setRuns(5000);
        t.getSettings().setSteps(INIT_SIZE * 2);
        t.getSettings().setSize(INIT_SIZE);
        Results rs = t.play();
        assertTrue(rs.getResults().get(0).getLostCfgs().size() > 0);
        Result res = rs.getResults().get(0);
        Configuration minCfg = res.getLostCfgs().get(0).getCfg();
        int min = INIT_SIZE;
        for (ICContext ctx : res.getLostCfgs()) {
            if (!ctx.getCfg().isOnesMajority() && ctx.getCfg().cardinality() < min) {
                minCfg = ctx.getCfg();
                min = ctx.getCfg().cardinality();
            }
        }
        SimpleCA ca = new SimpleCA();
        ca.setRule(rule);
        ca.setSize(INIT_SIZE);
        ca.setConfiguration(minCfg);

        assertTrue(!ca.getConfiguration().isOnesMajority());
        while (!ca.isStopped()) {
            ca.next();
        }
        assertTrue(ca.getConfiguration().isOnesMajority());

        ca.setSize(SIZE);
        Configuration c = new ConfigurationFactory(ConfigurationFactory.Implementation.BooleanArray).create(SIZE);
        for (int i = 0; i < INIT_SIZE; i++) {
            c.set(i, minCfg.get(i));
        }
        ca.setConfiguration(c);

        assertTrue(!ca.getConfiguration().isOnesMajority());
        while (!ca.isStopped()) {
            System.out.println(ca.getConfiguration().cardinality());
            ca.next();
        }
        assertTrue(ca.getConfiguration().isOnesMajority());
    }

    public void testZeroSets() {
        /**
         * 55   0110111
         * 103  1100111
         * 122  1111010
         */
        assertEquals(4, rule.getZeroSubsets().size());
        for (int x : rule.getZeroSubsets()) {
            System.out.println(Utils.toBinaryString(7, x));
        }
//        assertTrue(rule.isWashing());
//
//        rule.setBit(122, true);
//        assertTrue(rule.isWashing());
//
//        rule.setBit(120, true);
//        assertFalse(rule.isWashing());

    }

    public void testModifiedMonotony() throws Exception {
        Rule r = Utils.getRule(KnownCARepository.GKL);
        assertTrue(r.getBit(80));
        r.setBit(80, false);
        assertFalse(r.getBit(122));
        r.setBit(122, true);
        assertTrue(r.getBit(82));
        r.setBit(82, false);
        assertFalse(r.getBit(120));
        r.setBit(120, true);

        for (int x : r.getZeroSubsets()) {
            System.out.println(Utils.toBinaryString(7, x));
        }
//        System.out.println(r.getZeroSubsets().size());

//        System.out.println(r.getMonotony());
//        Result res = Utils.testRule(r);
//        System.out.println(res.getValue());

    }


}
