package ru.ifmo.ga.ca;

import junit.framework.TestCase;
import ru.ifmo.ga.ca.KnownCARepository;
import ru.ifmo.ga.ca.Rule;
import ru.ifmo.ga.ca.Utils;
import ru.ifmo.ga.ca.tournament.Results;
import ru.ifmo.ga.ca.tournament.Tournament;

import java.lang.reflect.Field;

/**
 * User: Yuri D. Bedny
 * Date: Apr 2, 2006
 * Time: 1:54:29 PM
 */

public class KnownCATest extends TestCase {

    private static final int SIZE = 149;
    private static final int RUNS = 30000;
    private static final int STEPS = 320;
    private static final int RADIUS = 3;
    private static final double DELTA = 0.01;

    // time = 125.391s

    private final Tournament t = new Tournament();

    public void setUp() {
        t.getSettings().setSize(SIZE);
        t.getSettings().setRuns(RUNS);
        t.getSettings().setSteps(STEPS);
    }

    private void assertEquals(String ruleStr, double v) throws Exception {
        Rule rule = new Rule();
        rule.setRadius(RADIUS);
        rule.parseRule(ruleStr);
        t.getRules().add(rule);

        Results res = t.play();
        assertEquals(1, res.getResults().size());
        assertEquals(v, res.getResults().get(0).getValue(), DELTA);
    }

    public void testGKL() throws Exception {
        assertEquals(KnownCARepository.GKL, 0.816);
    }

    public void testKoza() throws Exception {
        assertEquals(KnownCARepository.KOZA, 0.82326);
    }

    public void testGEP1() throws Exception {
        assertEquals(KnownCARepository.GEP_1, 0.82513);
    }

    public void testGEP2() throws Exception {
        assertEquals(KnownCARepository.GEP_2, 0.8255);
    }

    public void testGP1() throws Exception {
        assertEquals(KnownCARepository.JP_1, 0.851);
    }

    public void testGP2() throws Exception {
        assertEquals(KnownCARepository.JP_2, 0.86);
    }

    public void testCardinality() throws Exception {
        assertEquals(64, Utils.getRule(KnownCARepository.JP_2).cardinality());
        assertEquals(65, Utils.getRule(KnownCARepository.JP_1).cardinality());
        assertEquals(64, Utils.getRule(KnownCARepository.GEP_2).cardinality());
        assertEquals(64, Utils.getRule(KnownCARepository.GEP_1).cardinality());
        assertEquals(64, Utils.getRule(KnownCARepository.GKL).cardinality());
        assertEquals(64, Utils.getRule(KnownCARepository.KOZA).cardinality());
        assertEquals(64, Utils.getRule(KnownCARepository.MY_12).cardinality());

    }

    public void testMonotonous() throws Exception {
        assertEquals(101, Utils.getRule(KnownCARepository.JP_2).getMonotony());
        assertEquals(123, Utils.getRule(KnownCARepository.JP_1).getMonotony());
        assertEquals(25, Utils.getRule(KnownCARepository.GEP_2).getMonotony());
        assertEquals(25, Utils.getRule(KnownCARepository.GEP_1).getMonotony());
        assertEquals(9, Utils.getRule(KnownCARepository.GKL).getMonotony());
        assertEquals(18, Utils.getRule(KnownCARepository.KOZA).getMonotony());
        assertEquals(10, Utils.getRule(KnownCARepository.MY_12).getMonotony());
    }

    public void testWashing() throws Exception {
        assertFalse(Utils.getRule(KnownCARepository.JP_2).isWashing());
        assertFalse(Utils.getRule(KnownCARepository.JP_1).isWashing());
        assertFalse(Utils.getRule(KnownCARepository.GEP_2).isWashing());

        //todo!!
//        assertTrue(Utils.getKnownRule(KnownCARepository.GEP_1).isWashing());
//        assertTrue(Utils.getKnownRule(KnownCARepository.GKL).isWashing());
//        assertTrue(Utils.getKnownRule(KnownCARepository.KOZA).isWashing());
//        assertTrue(Utils.getKnownRule(KnownCARepository.MY_12).isWashing());
    }

    public void toCNF() throws Exception{
        System.out.println(Utils.getRule(KnownCARepository.JP_2).toWolframMathCNF(new char[]{'a','b','c','d','e','f', 'g'}));
    }


    public void testCorectness() throws Exception {
//        t.getSettings().setCorrectness(true);
        Rule rule = new Rule();
        rule.setRadius(RADIUS);
        rule.parseRule(KnownCARepository.GKL);
        t.getRules().add(rule);
        System.out.println(t.play());

    }

    public void diff() throws Exception {
        System.out.println(Utils.diffFromKnown("00000001000101000011000011010111000100010000111100111001010101110000010110110100111111110001011111110001001111011111100101010111"));
    }


 }
