package ru.ifmo.ga.ca;

import junit.framework.TestCase;
import ru.ifmo.ga.ca.Rule;
import ru.ifmo.ga.ca.Utils;
import ru.ifmo.ga.ca.SimpleCA;
import ru.ifmo.ga.ca.KnownCARepository;
import ru.ifmo.ga.ca.tournament.Result;
import ru.ifmo.ga.ca.tournament.ICContext;
import ru.ifmo.ga.ca.configuration.Configuration;
import ru.ifmo.ga.ca.configuration.ConfigurationFactory;

/**
 * User: Yuri D. Bedny
 * Date: May 20, 2006
 * Time: 3:32:24 PM
 */

public class RuleTest extends TestCase {

    public void testGreyRepresenation() throws Exception {
        System.out.println(Utils.getRule(KnownCARepository.JP_2));
        System.out.println(Utils.getRule(KnownCARepository.JP_2).toGreyTable());
    }

    public void testMonotonousRandom() {
        Rule r = new Rule();
        r.setRadius(3);
        for (int i = 0; i < 1000; i++) {
            r.randomize();
            assertFalse(r.isMonotonous());
        }
    }

    /**
     * It's very unprobably that random automaton is washing.
     */
    public void testWashingRandom() {
        Rule r = new Rule();
        r.setRadius(3);
        for (int i = 0; i < 1000; i++) {
            r.randomize();
            assertFalse(r.isWashing());
        }
    }

    public void testWash() throws Exception {
        final int SIZE = 1000;

        Rule r = new Rule();
        r.setRadius(2);
        r.parseRule("00000000011101110111011101110111");
        assertTrue(r.isMonotonous());
        assertTrue(r.isWashing());

        for (int x : r.getZeroSubsets()) {
            System.out.println(Utils.toBinaryString(r.getRadius() * 2 + 1, x));
        }
        SimpleCA ca = new SimpleCA();
        ca.setSize(SIZE);
        ca.setRule(r);
        Configuration c = new ConfigurationFactory(ConfigurationFactory.Implementation.BooleanArray).create(SIZE);
        Result res = Utils.testRule(r);
//        System.out.println("res.getValue() = " + res.getValue());
        for (ICContext ctx : res.getLostCfgs()) {
            c.clear();
            for (int i = 0; i < ctx.getCfg().size(); i++) {
                c.set(i, ctx.getCfg().get(i));
            }
            ca.setConfiguration(c);
            while (!ca.isStopped()) {
                ca.next();
            }
            assertEquals(0, ca.getConfiguration().cardinality());
        }
    }

    public void testMonotonicAndWashingR2() throws Exception {
        Rule r = new Rule();
        r.setRadius(2);
        r.parseRule("00000000000101110001011101110111");
        assertTrue(r.isMonotonous());
        assertTrue(r.isWashing());
    }

    public void testNonMonotonicWashing() throws Exception {
        Rule r = new Rule();
        r.setRadius(1);
        r.parseRule("00010100");
        assertFalse(r.isMonotonous());
        assertTrue(r.isWashing());

        r.parseRule("00000100");
        assertFalse(r.isMonotonous());
        assertTrue(r.isWashing());

        r.parseRule("01000100");
        assertFalse(r.isMonotonous());
        assertFalse(r.isWashing());
    }

    public void testMonotonous() throws Exception {
        Rule r = new Rule();
        r.setRadius(0);
        r.setBit(0, false);
        r.setBit(1, true);
        assertTrue(r.isMonotonous());

        r.setBit(0, true);
        assertTrue(r.isMonotonous());

        r.setBit(1, false);
        assertFalse(r.isMonotonous());

        r.setBit(0, false);
        assertTrue(r.isMonotonous());

        /**  cba
         * 0 000 0
         * 1 001 0                     !a && b && !c || a && b && !c || !a && b && c || a && b && c
         * 2 010 1   0
         * 3 011 1
         * 4 100 0
         * 5 101 0
         * 6 110 1 0
         * 7 111 1
         */
        r.setRadius(1);
        r.parseRule("00110011");
        assertTrue(r.isMonotonous());

        r.setBit(6, false);
        assertFalse(r.isMonotonous());

        r.setBit(2, false);
        assertTrue(r.isMonotonous());
    }


    public void testMinimizeCNF() throws Exception {//todo maybe move this test
        assertEquals("d'eg + cd + ad", Utils.minRuleExpr(KnownCARepository.GKL));

//        System.out.println(Utils.minRuleExpr(KnownCARepository.JP_1));
//        System.out.println(Utils.minRuleExpr(KnownCARepository.JP_2));
    }

    public void testWolframCNF() throws Exception {
        Rule r = new Rule();
        r.setRadius(0);
        r.setBit(0, true);
        r.setBit(1, false);

        assertEquals("!a", r.toWolframMathCNF(new char[]{'a'}).trim());

        r.setRadius(1);
        r.parseRule("00110011");
        assertEquals("!a && b && !c || a && b && !c || !a && b && c || a && b && c", r.toWolframMathCNF(new char[]{'a', 'b', 'c'}));
    }

    public void testParseHex() {
        Rule r = new Rule();
        r.setRadius(3);
        r.parseHex("0504058705000f77037755837bffb77f");
        System.out.println(r.toString());
        assertEquals(0.77, Utils.testRule(r, 1000).getValue(), 0.2);
    }
}
