package ru.ifmo.ga.ca;

import junit.framework.TestCase;
import ru.ifmo.ga.ca.Rule;
import ru.ifmo.ga.ca.SimpleCA;
import ru.ifmo.ga.ca.configuration.ConfigurationFactory;

import java.util.ArrayList;

/**
 * User: Yuri D. Bedny
 * Date: Apr 2, 2006
 * Time: 11:51:58 AM
 */

public class SimpleCATest extends TestCase {

    private ArrayList<ConfigurationFactory> getFactories() {
        ArrayList<ConfigurationFactory> f = new ArrayList<ConfigurationFactory>();
        for (ConfigurationFactory.Implementation impl : ConfigurationFactory.Implementation.values())
            f.add(new ConfigurationFactory(impl));
        return f;
    }

    public void testRule() throws Exception {
        Rule rule = new Rule();
        rule.setRadius(3);
        String s = "00000000010111110000000001011111000000000101111100000000010111110000000001011111111111110101111100000000010111111111111101011111";
        rule.parseRule(s);
        assertEquals(s, rule.toString());
        assertEquals(false, rule.getBit(24));
        assertEquals(true, rule.getBit(25));
        assertEquals(false, rule.getBit(26));
        assertEquals(false, rule.getBit(0));
        assertEquals(true, rule.getBit(127));

    }

    public void testNext() throws Exception {
        for (ConfigurationFactory f : getFactories()) {
            Rule rule = new Rule();
            rule.setRadius(1);
            String s = "10011110";
            rule.parseRule(s);
            assertEquals(s, rule.toString());
            SimpleCA ca = new SimpleCA();
            ca.setSize(10);
            ca.setRule(rule);
            String ic = "1100010101";
            f = new ConfigurationFactory(ConfigurationFactory.Implementation.BitSet);
            ca.setConfiguration(f.create(ic));
            assertEquals(ic, ca.toString());
            ca.next();
            assertEquals("0111001011", ca.toString());
        }
    }

}
