package ru.ifmo.ga.ca.randomizer;

import junit.framework.TestCase;
import ru.ifmo.ga.ca.randomizer.Randomizer;
import ru.ifmo.ga.ca.randomizer.RandomizerFactory;
import ru.ifmo.ga.ca.configuration.ConfigurationFactory;
import ru.ifmo.ga.ca.configuration.Configuration;

/**
 * User: Yuri D. Bedny
 * Date: Apr 2, 2006
 * Time: 8:37:41 PM
 */

public class RandomizerTest extends TestCase {

    private Randomizer get(RandomizerFactory.Imlementation impl, int size) {
        RandomizerFactory f = new RandomizerFactory(impl, ConfigurationFactory.Implementation.BitSet);
        return f.create(size);

    }

    public void testOverDensity() {
        final int N = 1000000;
        final int SIZE = 100;
        int d[] = new int[SIZE + 1];
        Randomizer rnd = get(RandomizerFactory.Imlementation.OverDensity, SIZE);
        for (int i = 0; i < N; i++) {
            Configuration c = rnd.random();
            d[c.cardinality()]++;
        }
        final double AVG = N * 1.0 / SIZE;
        final double EPS = 0.05;
        for (int i = 0; i < SIZE; i++) {
            assertTrue(Math.abs(d[i] * 1.0 - AVG) < EPS * AVG);
        }
    }


    /**
     *  N / 4  (13.125 sec)
     */
    public void testEveryBit() {
        final int N = 100000;
        final int T = 1000;
        final double DELTA = 5 * Math.sqrt(N) / (2 * N);// 5 times large then sqrt(d)
        Randomizer rnd = get(RandomizerFactory.Imlementation.EveryBit, N);
        for (int i = 0; i < T; i++) {
            Configuration ic = rnd.random();
            assertTrue(Math.abs(0.5 - ic.cardinality() * 1.0 / N) < DELTA);
        }
    }

    public void testConsequent() {
        final int M = 7;
        Randomizer rnd = get(RandomizerFactory.Imlementation.Consequent, M);
        Configuration c = rnd.random();
        int i = 0;
        while (c != null) {
            c = rnd.random();
            i++;
        }
        assertEquals(19, i);
    }

}
