package ru.ifmo.ga.ca.tree;

import junit.framework.TestCase;
import ru.ifmo.ga.ca.Utils;
import ru.ifmo.ga.ca.Rule;
import ru.ifmo.ga.ca.KnownCARepository;
import ext.bmin.MinimizedTable;

/**
 * User: Yuri D. Bedny
 * Date: May 29, 2006
 * Time: 2:11:47 AM
 */

public class ExpressionTest extends TestCase {
    public void testCalc() throws Exception {
        final char[] var = new char[]{'a', 'b', 'c', 'd', 'e', 'f', 'g'};
        final boolean [] val = new boolean[7];
        Rule r = Utils.getRule(KnownCARepository.GKL);
        MinimizedTable mt = new MinimizedTable(
                r.toCNF(var), null
        );
        assertEquals("d'eg + cd + ad", Utils.minRuleExpr(KnownCARepository.GKL));

        Expression expr = new Expression(mt);
        assertFalse(expr.isTruth());
        assertFalse(expr.isFalse());
        assertFalse(expr.calc(val));

        val[0] = true;//a
        assertFalse(expr.calc(val));

        val[1] = true;//b
        assertFalse(expr.calc(val));

        val[2] = true;//c
        assertFalse(expr.calc(val));

        val[3] = true;//d - cd
        assertTrue(expr.calc(val));

        val[2] = false;//c - ad
        assertTrue(expr.calc(val));

        val[3] = false;//d
        val[5] = true;//e
        assertFalse(expr.calc(val));

        val[4] = true; // e
        assertFalse(expr.calc(val));

        val[6] = true; // g - d'eg
        assertTrue(expr.calc(val));

        val[0] = false; //a
        val[3] = true; //d
        assertFalse(expr.calc(val));

        Expression ex = new Expression("a+a'");
        assertTrue(ex.isTruth());
        assertFalse(expr.isFalse());
        boolean[] b = new boolean[7];
        assertTrue(ex.calc(b));
        
    }

    public void testToRule() throws Exception {
        Rule rule = Utils.getRule(KnownCARepository.JP_2);
        Expression exp = new Expression(rule, new char[]{'a', 'b', 'c', 'd', 'e', 'f', 'g'});
        assertEquals(rule, exp.toRule());

        rule = Utils.getRule(KnownCARepository.JP_1);
        exp = new Expression(rule, new char[]{'a', 'b', 'c', 'd', 'e', 'f', 'g'});
        assertEquals(rule, exp.toRule());

    }
}
